from rest_framework import serializers

from ichec_django_core.models import Member, MemberIdentifier

from .core import SERIALIZER_BASE_FIELDS


class MemberIdentifierSerializer(serializers.ModelSerializer):

    # Needs explicit inclusion to use in Member list serializer updates
    id = serializers.IntegerField(required=False)

    class Meta:
        model = MemberIdentifier
        fields = ("id", "id_type", "value")


class MemberBaseSerializer(serializers.HyperlinkedModelSerializer):

    identifiers = MemberIdentifierSerializer(many=True)

    class Meta:
        model = Member
        fields = SERIALIZER_BASE_FIELDS + (
            "username",
            "email",
            "first_name",
            "last_name",
            "phone",
            "profile",
            "identifiers",
        )
        read_only_fields = SERIALIZER_BASE_FIELDS + ("profile",)

    def create(self, validated_data):
        ids = validated_data.pop("identifiers")

        instance = super().create(validated_data)
        MemberIdentifier.objects.bulk_create(
            [MemberIdentifier(member=instance, **eachId) for eachId in ids]
        )
        return instance

    def get_existing(self, id: str, collection: list):
        for c in collection:
            if c.id == id:
                return c
        raise RuntimeError("Requested non-existing entry")

    def update(self, instance, validated_data):
        incoming_ids = validated_data.pop("identifiers")

        instance = super().update(instance, validated_data)

        all_existing = instance.identifiers.all()

        # If any ids no longer exist delete them
        incoming_ids_ids = [
            incoming["id"] for incoming in incoming_ids if "id" in incoming
        ]
        existing_ids = [existing.id for existing in all_existing]
        for existing in existing_ids:
            if existing not in incoming_ids_ids:
                MemberIdentifier.objects.get(id=existing).delete()

        # If there is an id do an update, else do a create
        serializer = MemberIdentifierSerializer()
        for eachId in incoming_ids:
            if "id" not in eachId:
                MemberIdentifier.objects.create(member=instance, **eachId)
            else:
                serializer.update(self.get_existing(eachId["id"], all_existing), eachId)
        return instance


class MemberResponseSerializer(MemberBaseSerializer):

    profile = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="member_profiles"
    )
    profile_thumbnail = serializers.HyperlinkedIdentityField(
        read_only=True, view_name="member_profile_thumbnails"
    )

    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields + (
            "organizations",
            "profile_thumbnail",
        )
        read_only_fields = ("profile", "profile_thumbnail", "organizations")

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        if not instance.profile:
            rep["profile"] = None
        if not instance.profile_thumbnail:
            rep["profile_thumbnail"] = None
        return rep


class MemberDetailResponseSerializer(MemberResponseSerializer):

    permissions = serializers.SerializerMethodField("get_all_permissions")

    class Meta:
        model = Member
        fields = MemberResponseSerializer.Meta.fields + ("permissions",)
        read_only_fields = MemberResponseSerializer.Meta.read_only_fields + (
            "permissions",
        )

    def get_all_permissions(self, obj):
        return obj.get_all_permissions()


class MemberListSerializer(MemberBaseSerializer):
    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields
        read_only_fields = MemberBaseSerializer.Meta.read_only_fields

    def to_representation(self, instance):
        return MemberResponseSerializer(context=self.context).to_representation(
            instance
        )


class MemberDetailSerializer(MemberBaseSerializer):
    class Meta:
        model = Member
        fields = MemberBaseSerializer.Meta.fields
        read_only_fields = MemberBaseSerializer.Meta.read_only_fields

    def to_representation(self, instance):
        return MemberDetailResponseSerializer(context=self.context).to_representation(
            instance
        )



class RequestLoggerMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        print("Req path", request.path)
        print("Req host", request.get_host())
        print("Req Cookies", request.COOKIES)
        print("Req Headers", request.headers)

        response = self.get_response(request)

        print("Resp headers", response.headers)
        print("Resp cookies", response.cookies)
        print("Resp status", response.status_code)
        return response



from ichec_django_core.models import Member

from ichec_django_core.utils.test_utils.test_client import (
    AuthAPITestCase,
    setup_default_users_and_groups,
    add_group_permissions,
)


class MemberViewTests(AuthAPITestCase):

    template = {
        "username": "test_user",
        "identifiers": [{"id_type": "ORCID", "value": "1234-5678-1234-5678"}],
    }

    def setUp(self):
        self.url = "/api/members/"
        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Member,
            ["change_member", "add_member"],
        )

    def test_list_not_authenticated(self):
        self.assert_401(self.do_list())

    def test_detail_not_authenticated(self):
        self.assert_401(self.detail(1))

    def test_list_authenticated(self):
        self.assert_200(self.authenticated_list("regular_user"))

    def test_detail_authenticated(self):
        self.assert_200(self.authenticated_detail("regular_user", 1))

    def test_create_not_authenticated(self):
        data = {"username": "test_user"}
        self.assert_401(self.create(self.template))

    def test_create_authenticated_no_permission(self):
        self.assert_403(self.authenticated_create("regular_user", self.template))

    def test_create_authenticated(self):
        self.assert_201(self.authenticated_create("admin_user", self.template))

    def test_update_authenticated(self):
        response = self.authenticated_create("admin_user", self.template)
        self.assert_201(response)

        user = response.json()
        user["identifiers"][0]["value"] = "4321-5678-1234-5678"

        response = self.authenticated_update("admin_user", user["id"], user)
        self.assert_200(response)

        user_updates = response.json()
        self.assertTrue(
            user_updates["identifiers"][0]["value"] == "4321-5678-1234-5678"
        )

        user_updates["identifiers"].append({"id_type": "FREEFORM", "value": "1234"})
        response = self.authenticated_update("admin_user", user["id"], user_updates)
        self.assert_200(response)
        user_updates = response.json()
        self.assertTrue(len(user_updates["identifiers"]) == 2)

        user_updates["identifiers"] = []
        response = self.authenticated_update("admin_user", user["id"], user_updates)
        self.assert_200(response)

        user_updates = response.json()
        self.assertTrue(len(user_updates["identifiers"]) == 0)



from rest_framework import permissions

from .files import ObjectFileDownloadView, ObjectFileUploadView

from ..models import FormField, FormFieldValue


class FormFieldTemplateDownloadView(ObjectFileDownloadView):
    model = FormField
    file_field = "template"
    permissions_class = permissions.DjangoModelPermissions


class FormFieldTemplateUploadView(ObjectFileUploadView):
    model = FormField
    queryset = FormField.objects.all()
    file_field = "template"
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]


class FormFieldValueAssetDownloadView(ObjectFileDownloadView):
    model = FormFieldValue
    file_field = "asset"
    permissions_class = permissions.DjangoModelPermissions


class FormFieldValueAssetUploadView(ObjectFileUploadView):
    model = FormFieldValue
    queryset = FormFieldValue.objects.all()
    file_field = "asset"
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]



import unicodedata

from django.contrib.auth.models import Group

# from rest_framework.authtoken.models import Token
from mozilla_django_oidc.auth import OIDCAuthenticationBackend

from ..models.member import Member


def generate_username(email):

    return unicodedata.normalize("NFKC", email.split("@")[0][:150])


class MemberOIDCAuthenticationBackend(OIDCAuthenticationBackend):

    def __init__(self, *args, **kwargs):
        self.default_scopes = ("openid", "email", "profile")
        super().__init__(self, args, kwargs)

    def get_scopes(self):
        env_settings = self.get_settings("OIDC_RP_SCOPES", None)
        if env_settings:
            return env_settings.split()
        return self.default_scopes

    def verify_claims(self, claims):
        if "email" not in claims:
            return False
        return True

    def get_username(self, claims):
        if "preferred_username" in claims:
            return claims["preferred_username"]
        return generate_username(claims.get("email"))

    def create_user(self, claims):
        user = super().create_user(claims)
        user.first_name = claims.get("given_name", "")
        user.last_name = claims.get("family_name", "")

        # Create an associated Member
        member = Member(user_ptr_id=user.pk)
        member.__dict__.update(user.__dict__)

        # If there's a 'regular user' group assign them
        group = Group.objects.filter(name="regular_users").first()
        if group:
            member.groups.add(group)
        member.save()

        # Token.objects.create(user=user)

        return user



# Generated by Django 5.2.1 on 2025-11-18 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0002_memberidentifier"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formfieldvalue",
            name="value",
            field=models.CharField(blank=True, max_length=500),
        ),
    ]



# Generated by Django 5.2.1 on 2025-11-13 08:12

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="MemberIdentifier",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "id_type",
                    models.CharField(
                        choices=[("ORCID", "ORCID"), ("FREEFORM", "Freeform")],
                        max_length=8,
                    ),
                ),
                ("value", models.CharField(max_length=48)),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="identifiers",
                        to="ichec_django_core.member",
                    ),
                ),
            ],
        ),
    ]
